<?php

/*
Plugin Name: Calendar Archives
Plugin URI: http://www.sanisoft.com/blog/2009/08/21/wordpress-plugin-calendar-archives/
Description: Calendar Archives is a visualization plugin for your WordPress site which creates yearly calendar for your posts. Create a new page (having 'no sidebars' layout) for your calendar archive and insert the code [calendar-archive] in the editor. Load this page and enjoy the view!
Version: 1.0
Author: Amit Badkas
Author URI: http://www.sanisoft.com/blog/author/amitbadkas/
*/

/*  Copyright 2009  Amit Badkas  (email : amit@sanisoft.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class CalendarArchives
{
    function __construct()
    {
        // Call method to build everything needed by constructor
        $this->CalendarArchives();
    }

    function CalendarArchives()
    {
        // Plugin's directory
        $pluginDirectory = dirname(__FILE__) . DIRECTORY_SEPARATOR;

        // Path to cache generated HTML calendar
        $this->cachePath = $pluginDirectory . 'cached_pages' . DIRECTORY_SEPARATOR;

        // Path to cache background images
        $this->backgroundImagesCachePath = $pluginDirectory . 'cached_images' . DIRECTORY_SEPARATOR;

        // Add shortcode handler
        add_shortcode('calendar-archive', array(&$this, 'display'));

        // Add 'options' page
        add_action('admin_menu', array(&$this, 'adminMenu'));

        // Need to delete cache whenever post added/edited/deleted
        add_action('delete_post', array(&$this, 'deleteCache'));
        add_action('edit_post', array(&$this, 'deleteCache'));
        add_action('save_post', array(&$this, 'deleteCache'));
    }

    function activate()
    {
        // Need to build options while activatation
        $this->__getOptions();
    }

    function __getOptions()
    {
        // Create cache directory if not exists already
        if (!is_dir($this->cachePath) && @mkdir($this->cachePath))
        {
            chmod($this->cachePath, 0777);
        }

        // Create cache directory for images if not exists already
        if (!is_dir($this->backgroundImagesCachePath) && @mkdir($this->backgroundImagesCachePath))
        {
            chmod($this->backgroundImagesCachePath, 0777);
        }

        // Default options
        $default = array
        (
            'layout' => 1
            , 'reverse_months' => 'on'
            , 'hide_no_posts_months' => ''
            , 'cache' => (is_writeable($this->cachePath) ? 'on' : '')
            , 'show_images' => (is_writeable($this->backgroundImagesCachePath) ? 'on' : '')
        );

        // Get saved options
        $saved = get_option('CalendarArchives_options');

        // If possible, merge default and saved options to default one
        if (is_array($saved) && 0 < count($saved))
        {
            $default = array_merge($default, $saved);
        }

        // If saved options are different from merged default options then save merged default options as new one
        if ($saved != $default)
        {
            update_option('CalendarArchives_options', $default);
        }

        // Return merged default options
        return $default;
    }

    function deactivate()
    {
        // Delete plugin's options
        delete_option('CalendarArchives_options');
    }

    function plugin_action_links($links, $file)
    {
        // Plugin's file
        $pluginFile = basename(__FILE__);

        // If given file and plugin's file are same then prepend link of plugin's settings page
        if (basename($file) == $pluginFile)
        {
            array_unshift($links, '<a href="options-general.php?page=' . $pluginFile . '">Settings</a>');
        }

        // Return built links
        return $links;
    }

    function adminMenu()
    {
        // Add 'options' page to admin menu
        add_options_page('Calendar Archives Options', 'Calendar Archives', 8, basename(__FILE__), array(&$this, 'handleOptions'));
    }

    function handleOptions()
    {
        // If admin wants to remove cached images
        if (isset($_GET['remove_cached_images']) && (bool)$_GET['remove_cached_images'])
        {
            // Scan directory for cached images and remove them
            foreach (scandir($this->backgroundImagesCachePath) as $directoryContent)
            {
                if (is_file($this->backgroundImagesCachePath . $directoryContent))
                {
                    unlink($this->backgroundImagesCachePath . $directoryContent);
                }
            }

            // Remove references of 'remove cached images' flag from needed places and output success message
            $_SERVER['REQUEST_URI'] = str_replace('&remove_cached_images=1', '', $_SERVER['REQUEST_URI']);
            echo '<div class="updated fade"><p>Cached images have been removed successfully</p></div>';
            unset($_GET['remove_cached_images']);
        }

        // Get plugin's options
        $options = $this->__getOptions();

        // If form is submitted then save options
        if (isset($_POST['submitted']))
        {
            // Security check
            check_admin_referer('calendar-nonce');

            // Build options to save
            $options = array
            (
                'layout' => (int)$_POST['layout']
                , 'cache' => (string)$_POST['cache']
                , 'show_images' => (string)$_POST['show_images']
                , 'reverse_months' => (string)$_POST['reverse_months']
                , 'hide_no_posts_months' => (string)$_POST['hide_no_posts_months']
            );

            // Update options
            update_option('CalendarArchives_options', $options);

            // Delete cache
            $this->deleteCache();

            // Output success message
            echo '<div class="updated fade"><p>Plugin settings saved.</p></div>';
        }

        // Options form's action
        $actionUrl = $_SERVER['REQUEST_URI'];

        // Variables to use in form
        $layout = (int)$options['layout'];
        $cache = ($options['cache'] == 'on' ? 'checked' : '');
        $showImages = ($options['show_images'] == 'on' ? 'checked' : '');
        $reverseMonths = ($options['reverse_months'] == 'on' ? 'checked' : '');
        $hideNoPostsMonths = ($options['hide_no_posts_months'] == 'on' ? 'checked' : '');

        // Is cache directory writeable?
        $writeable = is_writeable($this->cachePath);

        // Is cache directory for images writeable?
        $writeableForImages = is_writeable($this->backgroundImagesCachePath);

        // Include options form
        include('calendar-archives-options.php');
    }

    function deleteCache()
    {
        // Remove all cache files
        foreach (glob($this->cachePath . '*.html') as $cacheFile)
        {
            unlink($cacheFile);
        }
    }

    function display()
    {
        global $wpdb;

        // If year is set in URL then get it from there
        if (isset($_GET['calendar_year']))
        {
            $year = (int)$_GET['calendar_year'];
        }

        // If year is not set/valid then use current one
        if (!isset($year) || $year > date('Y'))
        {
            $year = date('Y');
        }

        // Get plugin's options
        $options = $this->__getOptions();

        // Initialize variable to store cache flag
        $cache = (bool)$options['cache'];

        // If cache is enabled and cache file exists then use it to display output
        if ($cache && is_file($this->cachePath . $year. '.html'))
        {
            return file_get_contents($this->cachePath . $year. '.html');
        }

        // Get posts for given year
        $posts = $wpdb->get_results('SELECT * FROM ' . $wpdb->posts . ' WHERE post_status = "publish" AND post_password = "" AND post_type = "post" AND YEAR(post_date) = ' . $year . ' ORDER BY post_date ASC');

        // Initialize variable to store 'show images' flag
        $showImages = (bool)$options['show_images'];

        // Initialize variable used to store indexes for posts per day
        $postsPerDay = array();

        // Initialize variable used to store background images
        $backgroundImages = array();

        // Loop through list of posts to build 'posts per day' data
        for ($index = 0; $index < count($posts); $index++)
        {
            // Post's time
            $postTime = strtotime($posts[$index]->post_date);

            // Post's month
            $month = (int)date('m', $postTime);

            // If no posts for given month then initialize array for it
            if (!isset($postsPerDay[$month]))
            {
                $postsPerDay[$month] = array();
            }

            // If no background images for given month then initialize array for it
            if (!isset($backgroundImages[$month]))
            {
                $backgroundImages[$month] = array();
            }

            // Post's month
            $day = (int)date('d', $postTime);

            // If no posts for given month's given day then initialize array for it
            if (!isset($postsPerDay[$month][$day]))
            {
                $postsPerDay[$month][$day] = array();
            }

            // If no background image for given month's given day then initialize boolean flag false for it
            if (!isset($backgroundImages[$month][$day]))
            {
                $backgroundImages[$month][$day] = false;
            }

            // Build needed data
            $postsPerDay[$month][$day][] = $index;

            // If 'show images' flag is enabled and no background image for given month's given day then proceed further
            if ($showImages && false === $backgroundImages[$month][$day])
            {
                // Initialize variable used to store list of matched images as per provided regular expression
                $matches = array();

                // Get all images from post's body
                preg_match_all('/<\s*img [^\>]*src\s*=\s*[\""\']?([^\""\'>]*)/i', $posts[$index]->post_content, $matches);

                // If there are any image in post's body then proceed further
                foreach ($matches[1] as $backgroundImage)
                {
                    // Initialize variable used to store image's name
                    $backgroundImagename = $posts[$index]->ID . '-' . md5($backgroundImage);

                    // If image already downloaded then use it otherwise download it
                    if (is_file($this->backgroundImagesCachePath . $backgroundImagename))
                    {
                        // left blank intensionally
                    }
                    else if (false !== file_put_contents($this->backgroundImagesCachePath . $backgroundImagename, file_get_contents($backgroundImage)))
                    {
                        chmod($this->backgroundImagesCachePath . $backgroundImagename, 0777);
                    }
                    else
                    {
                        continue;
                    }

                    // Get image's information
                    $backgroundImageInformation = @getimagesize($this->backgroundImagesCachePath . $backgroundImagename);

                    // If image is valid and is not less than 140x140 then use it
                    if (false !== $backgroundImageInformation && 140 <= $backgroundImageInformation[0] && 140 <= $backgroundImageInformation[1])
                    {
                        $backgroundImages[$month][$day] = $backgroundImagename;
                        break;
                    }
                }
            }
        }

        // Get unique years for which posts are available
        $years = $wpdb->get_results('SELECT DISTINCT YEAR(post_date) AS year FROM ' . $wpdb->posts . ' WHERE post_status = "publish" AND post_password = "" AND post_type = "post" ORDER BY post_date DESC');

        // Grab output
        ob_start();
        include('calendar-layout-' . (int)$options['layout'] . '.php');
        $result = ob_get_contents();
        ob_end_clean();

        // If cache is enabled then cache generated output
        if ($cache)
        {
            // Cache file's path
            $cacheFile = $this->cachePath . $year. '.html';

            // Put generated output in cache file
            file_put_contents($cacheFile, $result);

            // Make cache file world writeable
            chmod($cacheFile, 0777);
        }

        // Return generated output
        return $result;
    }

    function getImageUrl($backgroundImage, $backgroundImageDimension)
    {
        // Background image path
        $backgroundImage = $this->backgroundImagesCachePath . $backgroundImage;

        // Get background image's information
        $backgroundImageInformation = @getimagesize($backgroundImage);

        // Get background image's extension
        $backgroundImageFileExtension = str_replace('image/', '', $backgroundImageInformation['mime']);

        // If background image's extension is not 'gif' or 'png' then it must be 'jpg'
        if (!in_array($backgroundImageFileExtension, array('gif', 'png')))
        {
            $backgroundImageFileExtension = 'jpg';
        }

        // Get background image's basename
        $backgroundImageBasename = basename($backgroundImage, '.' . $backgroundImageFileExtension);

        // If re-sized background image exists then use it
        if (is_file($this->backgroundImagesCachePath . $backgroundImageBasename . '-' . $backgroundImageDimension . '.' . $backgroundImageFileExtension))
        {
            $backgroundImage = $this->backgroundImagesCachePath . $backgroundImageBasename . '-' . $backgroundImageDimension . '.' . $backgroundImageFileExtension;
        }
        else
        {
            // Include needed library
            require_once ABSPATH . 'wp-admin' . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'image.php';

            // Re-size background image
            $backgroundImage = image_resize($backgroundImage, $backgroundImageDimension, $backgroundImageDimension, true, $backgroundImageDimension);

            // Make background image world writeable
            chmod($backgroundImage, 0777);
        }

        // Return background image's URL
        return str_replace(ABSPATH, get_option('siteurl') . '/', $backgroundImage);
    }
}

// Create plugin object
$CalendarArchives = new CalendarArchives();

// Register activation hook as plugin's activate() method
register_activation_hook(__FILE__, array(&$CalendarArchives, 'activate'));

// Register de-activation hook as plugin's deactivate() method
register_deactivation_hook(__FILE__, array(&$CalendarArchives, 'deactivate'));

// Add filter for plugin's action links
add_filter('plugin_action_links', array(&$CalendarArchives, 'plugin_action_links'), 10, 2);
